__doc__ = """\
Проект: IIGFS
Начато: Sun Sep 24 03:12:37 MSK 2000
Автор:  Andrey Orlov  (c) 2002
Верcия: $Id: libIIGFS.py 49951 2008-02-06 20:16:04Z cray $

Название:
        IIGFS / Библиотека вспомогательных функций
        
Описание:        
        Данная библиотека содержит общеупотребительные
        функции, используемые различными модулями. 
        За редким исключением, все функции включены в
        класс libIIGFS_mixin, который 
        использован как суперкласс во всех объектах IIGFS.

        Некотороые функции имееют лишь историческое
        значение и использовать их не рекомендуется.

Лицензия:
        GPL
        
Гарантия и отвественность :
        Никаких гарантий и ответственности

"""
__version__ = '$Revision: 49951 $'[10:-1]
# $Source: /var/lib/cvs/IIGFS/zope/Products/MOFS/libIIGFS.py,v $

debug = 0

import urllib
import string
import cgi
import re
from marshal import loads,dumps
from zlib import decompress,compress
from base64 import encodestring,decodestring
from Globals import MessageDialog
import DateTime 
trivial_trancetable = string.maketrans('','')
from AccessControl.SecurityManagement import getSecurityManager

from marshal import loads
from zlib import decompress
from urllib import unquote

#compress = lambda x : x
#decompress = lambda x : x

CantUnpack = 'Can not unpack data'

Unauthorized = 'Unauthorized'

class libIIGFS_mixin :
    """Библиотека полезных методов, отдаваемых в пространство имен Z-сервера
    В последствии нужно будет воткнуть библиотечку в dtml.

    """

    def request_setstart(self,seq,start_id='start_id',start='start',check=None) :
        """ Установить в запрсе поле start равным индексу элемента с
        идентификатором start_id в списке объектов, аргументы:

        seq - список объектов, который будет скролирован;

        start_id - имя переменной запроса, содержащей стартовый
        иденификатор;

        start - имя переменной запроса, в которую будет внесен стартовый
        индекс;

        Функция возвращает список объектов неизменым. """

        if self.REQUEST.has_key(start_id) :
            start_id = self.REQUEST[start_id] 
            for n in range(0,len(seq)) :
                if seq[n].getId() == start_id :
                        self.REQUEST[start] = n+1
                        break
        else :
            if check :
                raise 'NotFound', "Объект %s не существует" % start_id
        return seq

    def request_filter_keys(self,REQUEST,exclude=(),include=None,use_include=None) :
        """ Вернуть отфильтрованный список ключей запроса
        
        REQUEST - запроc,
        include - если не равно None и флаг use_include не равен 0, то в
                список войдут только перечисленные в этом атрибуте ключи,
        exclude - перечисленные ключи не войдут в список,
        use_include - если равен 0, то параметр include не будет
                использован.  
        """


        if include is None or (use_include==0) :
            include = REQUEST.form.keys()
        else :
            if type(include) is type('') :
                include = (include,)
            include = map(lambda x : x.split(':')[0], include)
            include = filter(REQUEST.form.has_key,include)

        if type(exclude) is type('') :
            exclude = (exclude,)

        exclude = map(lambda x : x.split(':')[0], exclude)
        return filter( lambda x,e=exclude : not (x in e), include)


    def request_filter_hash(self,REQUEST,exclude=[],include=None,use_include=None) :
        """ Вернуть элементы запроса, отфильтрованные request_filter_keys и
        преобразованные в хэш имя -> значения. Элементы типа record отдаются
        поэлементно как ключи вида <элемент>.<атрибут>.  Парамтры см. 
        request_filter_keys.  """

        d = {}
        for attr in self.request_filter_keys(REQUEST,exclude,include,use_include) :
                value = REQUEST.form[attr]
                if isinstance(value,ZPublisher.HTTPRequest.record) :
                        for a1,v1 in value.items() :
                                d['%s.%s:record' % (attr,a1)] = v1
                else :
                        d[attr] = value
        return d

    def request_filter_hidden(self,REQUEST,exclude=[],include=None,use_include=None) :
        """ Вернуть элементы запроса, отфильтрованные request_filter_keys и
        преобразованные в элементы ввода HIDDEN, парамтры см.
        request_filter_keys 
        """

        s = ''
        for a,v in self.request_filter_hash(REQUEST,exclude,include,use_include).items() :
            t = type(v)
            if  t is type(0) :
                s = s + '<input type=hidden name="%s:int" value="%i">' % (a,v)
            elif t is type([]) or t is type(()) :
                for vs in v :
                    s = s + '<input type=hidden name="%s:list" value="%s">' % (a,cgi.escape(str(vs),1))
            else :
                s = s + '<input type=hidden name="%s" value="%s">' % (a,cgi.escape(str(v),1))
        return s

    def request_filter_query(self,REQUEST,exclude=[],include=None,use_include=None) :
        """ Вернуть строку query, построенную из элементов запроса,
        возвращенных request_filter_keys
        """

        s = ''
        for a,v in self.request_filter_hash(REQUEST,exclude,include,use_include).items() :
            t = type(v)
            if  t is type(0) :
                s = s + '%s:int=%i&' % (a,v)
            elif t is type([]) or t is type(()) :
                for vs in v :
                    s = s + '%s:list=%s&' % (a,urllib.quote_plus(str(vs)))
            else :
                s = s + '%s=%s&' % (a,urllib.quote_plus(str(v)))

        return s

    def request_pack_hash(self,**kw) :
        """ Вернуть упакованный методом request_pack хэш аргументов """
        return self.request_pack(kw)

    def request_unpack_hash(self, query) :
        """ Вернуть результат распаковки query методом request_unpack. Если
        типа результата - хэш, то его ключи копируются в хеш form.
        """
        hash = self.request_unpack(query)
        if type(hash) == type({}) :
            for attr,value in hash.items() :
                self.REQUEST.form[attr] = value
                self.REQUEST[attr] = self.REQUEST.form[attr]
        return hash

    def request_pack(self,item) : 
        """ Вернуть параметр item, упакованный для передачи через запрос. 
        Обычный метод упаковки - marshal + zip + base64 + "нормализация"
        base64.   
        """

        s = dumps(item)
        z = compress(s)

        if len(z) >= len(s) :
            v = 's' + s
        else :
            v = 'z' + z

#       print "(len(z),len(s)) = ",len(z),len(s)
        return string.translate(encodestring(v),string.maketrans('/+','-:') ,'\012====')

    def request_unpack(self,query) :
        """ Вернуть распакованое значение параметра query, ранее
        запакованное методом request_pack
        """

        l=len(query)
        try :
            v = decodestring(
                        string.translate(
                            query,string.maketrans('-:','/+')
                        )
                        + ('=' * ((l+3)/4*4-l))
                )
            if v[0] == 'z' :
                return loads(decompress(v[1:]))
            else :
                return loads(v[1:])
        except :
            raise CantUnpack

    def request_ishere(self,url=None,right_margin=None,left_margin=None) :
        """ Проверить факт того, что объект соотвествующий данному url
        является контейнером запрашиваемого объекта, аргументы:

        url - путь к объекту, полученный absolute_url(1), если не указан -
                используется путь к текущему объекту,
        right_margin - количество незначащих элементов в конце url, если не
                указан - то все значащие,
        left_margin -  количество незначащих элементов в начале url, если не
                указан - то все значащие,

        Возвращает 1 при успешной проверке.
        """

        if url is None :
            url = self.absolute_url(1)

        ur = string.split(url,'/')
        if right_margin : ur = ur[0:-right_margin]
        if left_margin  : ur = ur[0: left_margin]

        vr = list(self.REQUEST.get('VirtualRootPhysicalPath',[''])[1:])
        qr = string.split(self.REQUEST.PATH_INFO,'/')[1:]
        if debug :
            print '------------------- request_ishere'
            print 'VR:',vr
            print 'UR,QR 1:',ur,qr

        if vr == ur[0:len(vr)] :
            ur = ur[len(vr):]

        if vr == qr[0:len(vr)] :
            qr = qr[len(vr):]

        if debug : print 'UR,QR 2:',ur,qr

        if qr[0:len(ur)] == ur :
            return 1

        return None

    def request_check(self,**kw) :
        """ Проверить переменные запроса на соответствие указанным
        регулярным выражениям.  Имена передаваемых аргументов совпадают с
        именами проверяемых переменных запроса.
        """

        for val,checks in kw.items() :
            if (len(checks) == 2) and (type(checks[0]) != type(())) and (type(checks[0]) != type([])) :
                checks = (checks,)
            for r,msg in checks :
                reg = re.compile(r[1:])
                if r[0]=='-' :
                    if reg.search(self.REQUEST[val]) : self.request_msg_append(val,msg)
                elif r[0]=='+' :
                    if not reg.search(self.REQUEST[val]) : self.request_msg_append(val,msg)
                else :
                    raise ValueError,"Invalid symbol %s in regexp of (%s,%s,%s): first symbol must be '-' or '+'" % (r[0],val,r,msg)
        return self.request_msg_check()


    def zsql_method_scroll(self,method,key='start',size=10,exclude=[],include=None,REQUEST=None,**kw) :
        """ Вернуть результаты вызова ZSQL метода, обработанные методом zsql_result_list. 
        
        При вызове ZSQL метода ему передаются параметры настройки прокрутки (start & size). Аргументы см.
        в zsql_result_scroll."""

        if REQUEST is not None :
            for arg,val in REQUEST.form.items() :
                kw[arg] = val

        kw['start'] = max(0,int(self.request_default(key,1)) - 1)
        kw['size']  = int(size)+1
        return self.zsql_result_scroll(
            apply(method,(),kw),
            key,size,exclude,include)

    def zsql_result_scroll(self,l=None,key='start',size=10,exclude=[],include=None,r=None) :
        """ Добавляет в список результатов l запроса mysql поля,
        ответственные за прокрутку списка:

        previous-sequence-start-number -- стартовый номер предыдущего блока
            последовательности или None;

        next-sequence-start-number -- стартовый номер следующего блока
            последовательности или None;

        previous-sequence-query -- запрос за предыдущей последовательностью;

        previous-sequence-query -- запрос за следующей последовательностью;

        Длина блока списка - size, переменная запроса используемая для
        прокрутки - key, параметры exclude & include аналогичны описанию
        методов request_filter* и используются при составлении ссылок
        дальше-раньше.

        Метод возвращает список хэшей.  
        """

        if not self.REQUEST.has_key(key) :
            start = 0
        else :
            start = max(int(self.REQUEST[key])-1,0)

        if (r is None) and (l is not None) :
            try :
                r = l.dictionaries()
            except AttributeError :
                r = l
        d = {}
        d['previous-sequence-start-number']       = None
        d['next-sequence-start-number']           = None
        d['previous-sequence-query']              = None
        d['next-sequence-query']                  = None
        d['previous-sequence-url']                = None
        d['next-sequence-url']                    = None

        qs = self.request_filter_query(self.REQUEST,exclude=[key]+list(exclude),include=list(include))
        if start > 0 :
            d['previous-sequence-start-number'] = max(start+1-size,1)
            d['previous-sequence-query'] = '%s%s=%u' % (qs,key,d['previous-sequence-start-number'])
            d['previous-sequence-url'] = self.REQUEST['URL1']+'?'+d['previous-sequence-query']
            
        if len(r) > 0 :
            if len(r) > size :
                d['next-sequence-start-number'] = start + 1 + size
                d['next-sequence-query'] = '%s%s=%u' % (qs,key,d['next-sequence-start-number'])
                d['next-sequence-url'] = self.REQUEST['URL1']+'?'+d['next-sequence-query']
                r = r[0:size]

            for a,v in d.items() :
                r[0][a] = v
                r[-1][a] = v

        return r

    marked_list = zsql_result_scroll
    marked_list_db = zsql_method_scroll

    def cookielist_unpack(self,prefix='fld') :
        # Вернуть распакованный список, сохраненный в куке с префиксом имени
        # fld. Для редактирования списка используется функция
        # cookielist_edit

        if self.REQUEST.has_key(prefix + '_list') :
            try :
                return self.request_unpack(self.REQUEST[prefix + '_list'])
            except CantUnpack :
                pass
        return []
        

    def cookielist_edit(self,prefix='fld',add=[],lifeperiod=None, path='/') :
        """ Редактировать список хешей, хранимых в куке c префиксов fld,
        аргументы:

        prefix - префикс переменных запроса, обслуживающих данный редактор,

        add - список добавляемых хешей,

        lifeperiod - время жизни кука, если не указано - то используется
        сессионный кук,

        path - путь при хранении кука (см. RFC),

        Возможные команды запроса:

        <PREFIX>_add - добавить элементы концу списка, список элементов
                должен быть передан в аргументе add,

        <PREFIX>_delete - удалить элементы из списка, список номеров
                удаляемых элементов передается в <PREFIX>_id, элементы
                нумеруются с нуля.

        <PREFIX>_first - переместить элементы в начало списка, список
                номеров удаляемых элементов передается в <PREFIX>_id,
                элементы нумеруются с нуля.

        <PREFIX>_clean - Очистить корзину полностью

        Метод заносит в сообщения запроса с ключом <PREFIX>_list сообщения о
        выполненных операциях.
        """

        l = self.cookielist_unpack(prefix)
        if self.REQUEST.has_key(prefix + '_add') :
            if type(add) is not type([]) :
                l.extend([add])
            else :
                l.extend(add)

        elif self.REQUEST.has_key(prefix + '_delete') :
            try :
                di = map(lambda x: int(x), self.REQUEST[prefix + '_id'])
            except KeyError :
                self.request_msg_append(prefix + '_list', 'Не выбрано ни одного элемента для удаления')
                return l

            di.reverse()
            try :
                for item in di :
                    del(l[item])
            except IndexError:
                self.request_msg_append(prefix + '_list', 'Попытка удаления несуществующего элемента')
                return self.cookielist_unpack(prefix)
    
        elif self.REQUEST.has_key(prefix + '_first') :
            try :
                di = map(lambda x: int(x), self.REQUEST[prefix + '_id'])
            except KeyError :
                self.request_msg_append(prefix + '_list', 'Не выбрано ни одного элемента для помещения в начало')
                return l
            
            try :
                n = 0
                for item in di :
                    l.insert(n,l[item])
                    n = n + 1
                    del(l[item+1])
            except IndexError :
                self.request_msg_append(prefix + '_list', 'Попытка помещения в начало ранее удаленных элементов')
                return self.cookielist_unpack(prefix)
	elif self.REQUEST.has_key(prefix + '_clear') :
	    l = []
        else :
            return l
            
        self.cookielist_set(l,prefix,lifeperiod,path)
        return l

    def cookielist_set(self,l,prefix='fld',lifeperiod=None,path='/') :
        """ Упаковать и установить кук со списком """
        cookie = self.request_pack(l)
        if debug : print 'Размер корзины',len(cookie) 
        if len(cookie) > 1024 :
                self.request_msg_append(prefix + '_list', 'Операция не была выполнена, т.к. корзина переполнена')
                return self.request_unpack(self.REQUEST[prefix + '_list'])

        d = {}
        if lifeperiod :
                d['expires'] = (DateTime.DateTime()+lifeperiod).rfc822()
        if path :
		if path[0] != '/' :
		   path = '/' + path
                d['path'] = path

	#print 'path',path
        apply(self.REQUEST.RESPONSE.setCookie,(prefix + '_list',cookie),d)

        return l


    _type_url_prefix = re.compile('^((http)|(ftp)|(mailto)|(int)|(javascript))[:/]*')
    _type_url_full = re.compile('^(((http)|(ftp)|(mailto))*.*\.)|(int|javascript).*$')

    def type_url_tunning(self,url='') :
        #""" Вернуть нормализованный URL. URL должен включать протоколы
        #http, ftp, mailto, int, javascript.  Если протокол отсутствует - по
        #умолчанию подставляется http:// """

        url = string.strip(url)
        if self._type_url_full.match(url) :
                if self._type_url_prefix.match(url) :
                        return url
                else :
                        return 'http://' + url

        return ''

    _type_email_check = re.compile('([A-Za-z_.0-9]+@([0-9A-Za-z_]+\.)+[0-9A-Za-z_]*)').search

    def type_email_tunning(self,email='') :
        # вернуть нормализованный email
        em = self.type_email_check(email)
        if em : 
                return em.group(0)
        return ''

    def type_email_check(self,email='') :
        return self._type_email_check(email)




### -*- coding: utf-8 -*- #############################################
#######################################################################
"""
Interfaces for the Zope 3 based ng.adapter.pager package

$Id: interfaces.py 49951 2008-02-06 20:16:04Z cray $
"""
__author__  = "Andrey Orlov"
__license__ = "GPL"
__version__ = "$Revision: 49951 $"

from zope.interface import Interface
from zope.schema import Field, Tuple, Int, Bool

class IPager(Interface):

    def setPagerParameters(revert=None, orphan=None,size=None) :
        """ Set Pager Parameters """
        
    def getOb(key) :
        pass
        
    def getKey(index) :
        pass

    have_before = Bool()

    befores = Tuple()

    beforeURLs = Tuple()

    before  = Field()

    beforeURL = Field()

    chunk   = Tuple()

    keys    = Tuple()

    index   = Int()

    after   = Field()

    afterURL = Field()

    afters  = Tuple()        

    afterURLs  = Tuple()        

    have_after = Bool()

    len     = Int()

class IPagerSource(Interface) :
    def __getitem__(key) :
        """ Return object by key """
        
    def keys() :        
        """ Return ordered key sequence """
        


### -*- coding: utf-8 -*- #############################################
#######################################################################
""" OrderedContainer Adapter to IPagerSource Interface

$Id: container2pagersourceadapter.py 49951 2008-02-06 20:16:04Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 49951 $"

from zope.interface import implements,Interface
from zope.component import adapts
from interfaces import IPagerSource
from zope.cachedescriptors.property import CachedProperty
from zope.app.container.interfaces import IContainer

class Container2PagerSourceAdapter(object):
    __doc__ = __doc__

    implements(IPagerSource)
    adapts(IContainer)

    def __init__(self, context):
        self.context = context
       
    def keys(self) :
        return list(self.context.keys())
        
    def __getitem__(self,key) :
        return self.context[key]               
    



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Product class for the Zope 3 based product package

$Id: ianytitleadapter.py 49001 2007-12-24 13:29:26Z antel $
"""
__author__  = "Elena Antusheva, 2008"
__license__ = "GPL"
__version__ = "$Revision: 49001 $"

import sys
import re
from interfaces import IAnyTitle
from zope.interface import implements
from zope.component import adapts
from zope.app.container.interfaces import IContained
from zope.component import subscribers

class AnyTitleAdapter(object) :
    implements(IAnyTitle)
    adapts(IContained)

    shorttitlelength = 18
    
    subword = " ... "
    
    lensubword = len(subword)    

    def __init__(self, ob) :
        self.context = ob
        
    @property
    def title(self) :
        for adapter in sorted (subscribers([self.context], IAnyTitle)) :
            title = adapter.title
            if title is not None :
                title = title.strip()
                if title :
                    return adapter.title
        return IContained(self.context).__name__
    
    @property
    def shorttitle(self) :

        titles = ( y for y in ( x.title.strip() for x in subscribers([self.context], IAnyTitle) ) if y )
        
        try :
            title = titles.next()
        except StopIteration :
            title = IContained(self.context).__name__
        else :            
            for mtitle in titles :
                if (    (len(title) < len(mtitle) and len(mtitle) < self.shorttitlelength) 
                     or (len(title) > len(mtitle) and len(mtitle) > self.shorttitlelength) 
                     or (len(mtitle) < self.shorttitlelength < len(title))     ):
                    title = mtitle

        if len(title) < self.shorttitlelength : 
            return title

        title,flag = re.subn("\s[^\s]{%s,%s}\s" % ( 
                        len(title)-self.shorttitlelength+self.lensubword, 
                        len(title)-self.shorttitlelength+self.lensubword+3 )," ... ",title,1)

        if flag : 
            return title
        else :
            return ( title[:(self.shorttitlelength-self.lensubword)/2] 
                + self.subword
                + title[(len (title) - (self.shorttitlelength-self.lensubword)/2):] )




### -*- coding: utf-8 -*- #############################################
#######################################################################
""" mtimeAdapter that adapt IPersistent interface to
    IMTime interface

$Id: product.py 12897 2007-11-10 15:32:08Z cray $
"""
__author__  = "Yegor Shershnev"
__license__ = "GPL"
__version__ = "$Revision: 12897 $"

from zope.interface import implements
from zope.component import adapts
from persistent.interfaces import IPersistent
from interfaces import IMTime
import time

class mtimeAdapter(object):
    __doc__ = __doc__
    implements(IMTime)
    adapts(IPersistent)

    def __init__(self, context):
        self.context = context

    @property
    def mtime(self):
        if IPersistent(self.context)._p_changed:
            return time.time()
        else:
            return IPersistent(self.context)._p_mtime

    def strftime(self,format=u"%Y-%m-%d %H:%M:%S") :
        """ Return datetime as string using format argument """
        return time.strftime(format,time.localtime(self.mtime))
        


### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Product class for the Zope 3 based product package

$Id: product.py 12897 2007-11-10 15:32:08Z cray $
"""
__author__  = "Yegor Shershnev"
__license__ = "GPL"
__version__ = "$Revision: 12897 $"

from zope.interface import implements
from interfaces import IPath

class PathAdapterBase(object) :

    implements(IPath)
    
    def __init__(self, context) :
        self.context = context

    @property
    def tailpath(self) :
       return self._tailpath((self.name,))
       
       


### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Product class for the Zope 3 based product package

$Id: registrynamespace.py 49088 2007-12-29 23:27:21Z cray $
"""
__author__  = "Elena Antusheva, 2008"
__license__ = "GPL"
__version__ = "$Revision: 49088 $"

from zope.interface import Interface
from zope.interface import implements,implementedBy
from zope.app.file.image import Image
from zope.traversing.interfaces import ITraverser, ITraversable
from zope.app.catalog.interfaces import ICatalog
from ng.app.registry.interfaces import IRegistry
from zope.app.zapi import getUtilitiesFor

from zope.app.zapi import getUtility

class RegistryNamespace(object) :
    implements(ITraversable)

    def __init__(self,context,request) :
        self.context = context
        self.request = request

    def traverse(self,name,ignored) :
        nm = name.split(':')
        try:
            return ( y for x,y in getUtilitiesFor(IRegistry,context=self.context)).next().param(*nm)
        except StopIteration,msg:
            try :
                return nm[1]
            except IndexError :
                return ignored



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Adapter to IRecordSize interface which provide object record size in database

$Id: recordsize.py 49951 2008-02-06 20:16:04Z cray $
"""
__author__  = "Andrey Orlov,2008"
__license__ = "GPL"
__version__ = "$Revision: 49951 $"

from interfaces import IRecordSize 
import cPickle
import tempfile
from zope.location.interfaces import ILocation

class ReferenceCut(object) :

    def __init__(self, location):
        self.location = location
        self.pids_by_id = {}
        self.others_by_pid = {}
        self.load = self.others_by_pid.get

    def id(self, ob):
        if ILocation.providedBy(ob):
            if self.location is not ob :
                if id(ob) in self.pids_by_id:
                    return self.pids_by_id[id(ob)]
                pid = len(self.others_by_pid)
                pid += 1

                self.pids_by_id[id(ob)] = pid
                self.others_by_pid[pid] = ob
                return pid
                
        return None

class RecordSizeAdapter(object) :
    def __init__(self,context) :
        self.context = context

    @property        
    def size(self) :
      tmp = tempfile.TemporaryFile()
      persistent = ReferenceCut(self.context)

      # Pickle the object to a temporary file
      pickler = cPickle.Pickler(tmp, 2)
      pickler.persistent_id = persistent.id
      pickler.dump(self.context)

      # Now load it back
      return tmp.tell()



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""The adapter Contained2TitleAdapter that adopt IAttributeAnnotatable
   interface to ITitle interface

$Id: product.py 12897 2007-11-10 15:32:08Z cray $
"""
__author__  = "Yegor Shershnev"
__license__ = "GPL"
__version__ = "$Revision: 12897 $"

from zope.component import adapts
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.dublincore.interfaces import IZopeDublinCore
from contained2titleadapter import Contained2TitleAdapter

class ZDC2TitleAdapter(Contained2TitleAdapter) :

    adapts(IAttributeAnnotatable)
    
    @property
    def title(self) :
        title = IZopeDublinCore(self.context).title
        if not title :
            try:
                return super(ZDC2TitleAdapter, self).title
            except TypeError:
                return title
        
        return title



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""
Interfaces for the Zope 3 based ng.adapter.mtime package

$Id: product.py 12897 2007-11-10 15:32:08Z cray $
"""
__author__  = "Yegor Shershnev,2007"
__license__ = "GPL"
__version__ = "$Revision: 12897 $"

from zope.interface import Interface
from zope.schema import Float,TextLine

class IMTime(Interface):

    mtime = Float(
        title=u"Modification time",
        description=u"Last modification time in second from Unix Epoch"
        )

    def strftime(format=TextLine(title=u"TimeDate Format")) :
        """ Return datetime as string using format argument """
        return time.strftime("%Y-%m-%d %H:%M:%S",time.localtime(0))
        


### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Interfaces for the Zope 3 based ng.adapter.title package

$Id: interfaces.py 49951 2008-02-06 20:16:04Z cray $
"""
__author__  = ""
__license__ = "GPL"
__version__ = "$Revision: 49951 $"
 
from zope.interface import Interface
from zope.schema import TextLine
                
                
class ITitle(Interface) :
    """ Interface ITitle
    
        title - title of object
    """

    title = TextLine(title=u"Title",
                     description=u"Title of object")



### -*- coding: utf-8 -*- #############################################
#######################################################################
""" Zope.app.catalog.ResultSet Adapter to IPagerSource Interface

$Id: resultset2pagersourceadapter.py 49951 2008-02-06 20:16:04Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 49951 $"

from zope.interface import implements,Interface
from zope.component import adapts
from interfaces import IPagerSource

class List(list) :
    def __getitem__(self,index) :
        return super(List,self).__getitem__(int(index))

    def index(self,index) :
        return super(List,self).index(int(index))

class ResultSet2PagerSourceAdapter(object):
    __doc__ = __doc__

    implements(IPagerSource)

    def __init__(self, context):
        self.context = context
       
    def keys(self) :
        return List(self.context.uids)
        
    def __getitem__(self,key) :
        return self.context.uidutil.getObject(key)
    




### -*- coding: utf-8 -*- #############################################
#######################################################################
""" IRequest Adapter to IRequestExt interface

$Id: requestext.py 49951 2008-02-06 20:16:04Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 49951 $"

from zope.interface import Interface
from zope.interface import implements,implementedBy
from zope.publisher.browser import Record

import urllib
import string
import cgi
import re
from marshal import loads,dumps
from zlib import decompress,compress
from base64 import encodestring,decodestring
from datetime import datetime
import time

from interfaces import IRequestExt

class CantUnpack(Exception):
    """Can't unpack data"""


class RequestExt(object) :
    implements(IRequestExt)

    exclude=()
    include=None
    use_include=False

    def __init__(self, request):
        super(RequestExt, self).__init__(request)
        self.request = request

    def setParameters(self, exclude=(), include=None, use_include=False) :
        """
            Set filter parameters:
            include -- if not equal None and use_include flag is True, then list
                will be content only keys from include;
            exclude -- this keys will be excluded from list anyway;
            use_include -- if equal False, then parameter include can't be used. 
        """
        
        if type(exclude) is type('') :
            self.exclude = (exclude,)
        else :
            self.exclude = exclude
            
        if type(include) is type('') :
            self.include = (include,)
        else :            
            self.include = include
        self.use_include = use_include

    def keys(self) :
        """ Return list of request keys after filering """

        if self.include is None or (self.use_include == 0) :
            include = self.request.form.keys()
        else :
            include = [y for y in 
                        [ x.split(':')[0] for x in self.include ] 
                        if self.request.form.has_key(y) ]

        exclude = [ x.split(':')[0] for x in self.exclude]
        return [ x for x in include if not x in exclude ]

    def _parseRecordElement(self, attr, value):
        d = {}
        t = type(value)

        if isinstance(value, Record) :
            for a1,v1 in value.items() :
                d['%s.%s:record' % (attr, a1)] = v1
        else:
            d[attr] = value

        return d

    def _parseSequenceElement(self, attr, value, postfix=''):

        l = []
        t = type(value)
        newPostfix = ''
        if t == type([]):
            newPostfix = ':list' + postfix
        elif t == type(()):
            newPostfix = ':tuple' + postfix

        if newPostfix:
            for v1 in value:
                l.append(self._parseElement(attr, v1, newPostfix))

        return l

    def _parseElement(self, attr, value, postfix=''):
        t = type(value)
        newPostfix = ''
        if t == type(0):
            newPostfix = ':int'
        elif t == type(0.8):
            newPostfix = ':float'
        elif t == type(99999999999999):
            newPostfix = ':long'
        elif t == type(''):
            newPostfix = ':string'

        return (attr + newPostfix + postfix, value)

    def dict(self) :
        """ Return request items as dict key->value """
        
        d = {}
        for attr in self.keys() :
                value = self.request.form[attr]
                d.update(self._parseRecordElement(attr, value))

        return d

    def list(self):
        """ Return request items as dictionary list """
        l = []
        for a,v in self.dict().items() :
            l.extend(self._parseSequenceElement(a, v) or [self._parseElement(a, v)])

        return l

    def hidden(self,**kw) :
        """ Return request items as hidden HTML input tags"""

        s = ''
        for a,v in self.list() +kw.items():
            s = s + '<input type=hidden name="%s" value="%s">' % (a,cgi.escape(str(v),1))
        return s

    def query(self,**kw) :
        """ Return query string """
        s = ''
        for a,v in self.list() +kw.items():
            s = s + '%s=%s&' % (a,urllib.quote_plus(str(v)))

        return s

    def pack_dict(self,**kw) :
        """ Вернуть упакованный методом pack хэш аргументов """
        return self.pack(kw)

    def unpack_dict(self, query) :
        """ Вернуть результат распаковки query методом unpack. Если
        тип результата - словарь, то его ключи копируются в словарь form.
        """
        hash = self.unpack(query)
        if type(hash) == type({}) :
            for attr,value in hash.items() :
                self.request.form[attr] = value
        return hash

    def pack(self, item) :
        """ Вернуть параметр item, упакованный для передачи через запрос.
        Обычный метод упаковки - marshal + zip + base64 + "нормализация"
        base64.
        """

        s = dumps(item)
        z = compress(s)

        if len(z) >= len(s) :
            v = 's' + s
        else :
            v = 'z' + z

        return string.translate(encodestring(v),string.maketrans('/+','-:') ,'\012====')

    def unpack(self,query) :
        """ Вернуть распакованое значение параметра query, ранее
        запакованное методом pack
        """
        l=len(query)
        try :
            v = decodestring(
                        string.translate(
                            query,string.maketrans('-:','/+')
                        )
                        + ('=' * ((l+3)/4*4-l))
                )
            if v[0] == 'z' :
                return loads(decompress(v[1:]))
            else :
                return loads(v[1:])
        except :
            raise CantUnpack

class RequestExtView(RequestExt) :
    implements(IRequestExt)

    def __init__(self, context, request):
        self.context = context
        self.request = request

#
#        self.request.response.setHeader('Pragma', 'no-cache')
#        self.request.response.setHeader('Cache-Control', 'no-cache')
#        self.request.response.setHeader('Expires',
#            time.strftime("%a, %d %b %Y %H:%M:%S GMT",time.gmtime(time.time()))
#            )
#        self.request.response.setHeader('Last-Modified',
#            time.strftime("%a, %d %b %Y %H:%M:%S GMT",time.gmtime(time.time()))
#            )
#


### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Product class for the Zope 3 based product package

$Id: product.py 12897 2007-11-10 15:32:08Z cray $
"""
__author__  = "Yegor Shershnev"
__license__ = "GPL"
__version__ = "$Revision: 12897 $"

from zope.interface import Interface
from zope.component import adapts
from pathadapterbase import PathAdapterBase

class Any2PathAdapter(PathAdapterBase) :

    adapts(Interface)
    
    path = ""

    titledpath = ""

    tailpath = ()

    name = ""
    
    def _tailpath(self,path) :
        return path